﻿using DoubleDataGenerate.Core.Model;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SQLite;
using RemoteSQLite;
using AutoMapper;
using MahApps.Metro.IconPacks;
using System.Data;
using System.Threading;
using Misc;

namespace DoubleDataGenerate.Core
{
    public class DDGen : INotifyPropertyChanged, IDDGenService
    {
        public const string Gage0Title = "设备0";
        public const string Gage1Title = "设备1";


        private NLog.ILogger logger = NLog.LogManager.GetCurrentClassLogger();
        private static DDGen instance=null;
        public static DDGen Instance
        {
            get {
                if (instance == null)
                {
                    instance = new DDGen();
                    instance.Init();
                    instance.Start();
                }
                return instance;
            }
        }
    
        public event PropertyChangedEventHandler PropertyChanged;

#region 参数
        /// <summary>
        /// 前机设备
        /// </summary>
        public string Addr0 { get; set; } = "192.168.0.1:20000";
        /// <summary>
        /// 后机设备
        /// </summary>
        public string Addr1 { get; set; } = "127.0.0.1:20000";

        /// <summary>
        /// 前后机距离，单位m
        /// </summary>
        public double Distance { get; set; } = 40;

        /// <summary>
        /// 最小线速度 2m/min
        /// </summary>
        public double MinVelocity { get; set; } = 3;

        /// <summary>
        /// 找位置的时间差，不能大于60s
        /// </summary>
        public int SecondTolerance { get; set; } = 60;
        private int mix = 6;
        /// <summary>
        /// 混合数
        /// </summary>
        public int Mix 
        {
            get { return mix; }
            set {
                if (value< 1)
                    value = 1;

                if (mix != value)
                    mix = value;
            }
                
        }

        /// <summary>
        /// 轮询后机数据库周期！！！ 单位min 默认1，最小值0.5
        /// </summary>
        public int IntervalMin { get; set; } = 1;

#endregion
        #region 状态
        public bool IsError { get; private set; }
        /// <summary>
        /// 错误信息
        /// </summary>
        public string ErrMsg { get; private set; }

        /// <summary>
        /// 状态信息
        /// </summary>
        public string Msg { get; private set; }

        /// <summary>
        /// 剩余秒数
        /// </summary>
        public int SurplusSec { get; private set; }

        public long LastScanDataId { get; private set; } = -1;
        public DateTime LastScanDataTime { get; private set; }

        public bool IsRunning { get; private set; }

        public TimeSpan DistanceTs { get; private set; }

        /// <summary>
        /// 设备0 连接状态
        /// </summary>
        public bool IsConnected0 { get; private set; }

        /// <summary>
        /// 设备1 连接状态
        /// </summary>
        public bool IsConnected1 { get; private set; }

        #endregion

        RemoteSQLite.SQLiteRemoteServiceClient sqliteRemoteService0;
        RemoteSQLite.SQLiteRemoteServiceClient sqliteRemoteService1;

        List<Db_Profile> db_Profiles0;
        List<Db_Profile> db_Profiles1;
        List<Lc_BoltmapData> lc_BoltmapDatas1;

        List<ScanDataDiffPack> scanDataDiffPacks;
        DiffDbModel dbModel;

        string dbpath = @"D:\flydata\history.sqlite3";
        string ConnectString
        {
            get
            {
                return $"Data Source={dbpath};Version=3;";
            }
        }

        System.Threading.CancellationTokenSource cancellationTokenSource;

        string commandText;
        int timeOutMs = 1000;
        public void Init()
        {
            Load();
            //SurplusSec = GetSurplusSec();

            sqliteRemoteService0 = new RemoteSQLite.SQLiteRemoteServiceClient("SQLiteRemote", false, Addr0);
            
            sqliteRemoteService1 = new RemoteSQLite.SQLiteRemoteServiceClient("SQLiteRemote", false, Addr1);


            dbModel = new DiffDbModel(@"D:\flydata\diff_history.sqlite3");
            dbModel.Init();
            KeepDBSize(6);

            Misc.BindingOperations.SetBinding(sqliteRemoteService0, nameof(sqliteRemoteService0.IsConnected), this, nameof(IsConnected0));
            Misc.BindingOperations.SetBinding(sqliteRemoteService1, nameof(sqliteRemoteService1.IsConnected), this, nameof(IsConnected1));

        }


        public void KeepDBSize(int month)
        {
            if (month <= 3)
                month = 3;

            DateTime del_time = DateTime.Now - TimeSpan.FromDays(month * 30);
            string del_time_str = del_time.ToStringOfSQLiteFieldType();

            List<string> sqls = new List<string>();

            
            sqls.Add(
                $"DELETE FROM {dbModel.TbProfile.TableName}" +
                $" WHERE EndTime < {del_time_str}");
            sqls.Add(
                $"DELETE FROM {dbModel.TbDiffData.TableName}" +
                $" WHERE Time < {del_time_str}");

            dbModel.sqliteHelper.QueryTran(sqls);
        }


        int GetSurplusSec()
        {
            if (IntervalMin > 0)
                return (int)(IntervalMin * 60);
            else
                return 30;
        }

        bool InitDb()
        {
            
            return true;
        }
        public void Start()
        {
            if (IsRunning)
                return;

            if (!InitDb())
            {
                IsError = true;
                return;
            }
            IsRunning = true;
            cancellationTokenSource = new System.Threading.CancellationTokenSource();


            Task.Factory.StartNew(() =>
            {
                while (!cancellationTokenSource.IsCancellationRequested)
                {
                    int totalsec = 10;// GetSurplusSec() -1;
                    for (int i = 0; i < totalsec; i++)
                    {
                        SurplusSec = totalsec - i;
                        try
                        {
                            Task.Delay(1000, cancellationTokenSource.Token).Wait();
                        }
                        catch (TaskCanceledException e)
                        {
                            //关闭
                            return;
                        }
                    }
                    
                    try
                    {
                        OnPoll().Wait();
                    }
                    catch (Exception e)
                    {

                        ErrMsg = e.Message;
                        IsError = true;
                        logger.Fatal(e);
                    }
                }
            });
        }

        class MinMaxID 
        {
            public long? MinID { get; set; }
            public long? MaxID { get; set; }
        }

        
        CancellationTokenSource executeReaderCancellation;
        ExecuteReaderReponse executeReaderReponse;
        object executeReaderLocker = new object();

        /// <summary>
        /// 不允许重入
        /// </summary>
        /// <param name="sQLiteRemoteService"></param>
        /// <param name="commandText"></param>
        /// <returns></returns>
        async Task<DataTable> ExecuteReader(SQLiteRemoteServiceClient sQLiteRemoteService, string commandText) 
        {

            executeReaderCancellation = new CancellationTokenSource();
            sQLiteRemoteService.ExecuteReader(
                ConnectString, commandText, timeOutMs,
                (object asyncContext, object retData) =>
                {

                    executeReaderReponse = retData as ExecuteReaderReponse;
                    executeReaderCancellation.Cancel();
                }, null);
            
            try
            {
                await Task.Delay(timeOutMs, executeReaderCancellation.Token);
            }
            catch (Exception e)
            {
                //被叫醒,收到数据了
                if (!string.IsNullOrEmpty(executeReaderReponse.ErrMsg))
                {
                    //服务器发生异常， 客户端这里也抛出异常
                    throw new Exception(executeReaderReponse.ErrMsg);;
                }
                else
                {
                    //正常返回
                    return executeReaderReponse.Ret;
                }
            }
            //超时
            throw new TimeoutException();
        }
        async Task<DataTable> ExecuteReader2(SQLiteRemoteServiceClient sQLiteRemoteService, string commandText, string msg) 
        {
            DataTable dataTable;
            try
            {
                dataTable = await ExecuteReader(sQLiteRemoteService, commandText);
            }
            catch (Exception e)
            {
                IsError = true;
                ErrMsg = $"{msg} 数据异常: {commandText} {e.Message}";
                logger.Error(ErrMsg);
                return null;
            }

            if (dataTable.Rows.Count == 0)
            {
                IsError = true;
                ErrMsg = $"{msg} 数据异常:没有任何数据";
                logger.Error(ErrMsg);
                return null;
            }
            return dataTable;
        }
        async Task OnPoll()
        {
            if (!IsConnected0) {
                IsError = true;
                ErrMsg = $"{Gage0Title} 连接断开";
            }
            if (!IsConnected1)
            {
                IsError = true;
                ErrMsg = $"{Gage1Title} 连接断开";
            }
            //获取后机数据
            bool ret = await OnPoll_getScanData1();
            if (!ret)
                return;

            //获取前机数据
            ret = await OnPoll_getScanData0();
            if (!ret)
                return;

            //求差分数据
            OnPoll_calDiff();


            //最后再获取 profile数据
            if (!(await OnPoll_getProfile()))
                return;

            //数据保存到数据库
            OnPoll_save();

            IsError = false;
            Msg = $"生成数据成功，最后一条{Gage1Title} ScanData.ID = {LastScanDataId}";
        }

        async Task<bool> OnPoll_getScanData1() 
        {
            DataTable dataTable;
            DateTime dt = new DateTime(2020, 08, 04, 17, 14, 41);//DateTime.Now
            DateTime beginTime = dt - TimeSpan.FromMinutes(IntervalMin * 3);

            //---------------------------------------------------------------------------------------------
            //读取 时间大于 (当前时间-轮询时间) 的 Db_ScanData 数据列， 太旧的数据不取！！！
            commandText = $"SELECT MIN([ID]) AS [MinID],MAX([ID]) AS [MaxID] FROM ScanData WHERE [Time] >= { beginTime.ToStringOfSQLiteFieldType()} ORDER BY [ID]";

            dataTable = await ExecuteReader2(sqliteRemoteService1, commandText, $"获取{Gage1Title}");
            if (dataTable == null)
                return false;

            var mmid = SQLite.SQLiteHelper.ToObjs<MinMaxID>(dataTable).First();

            if (mmid.MinID == null) {
                //没有数据
                IsError = false;
                Msg = $"获取{Gage1Title} 数据成功, 但没有数据更新";
                return false;
            }
            long fromId = (long)mmid.MinID;


            //从本地数据库，获取最后一条 Db_DiffData
            var db_DiffDatas = dbModel.TbDiffData.Last(1);
            if (db_DiffDatas.Count() == 0)
            {
                LastScanDataId = -1;
            }
            else {
                LastScanDataId = db_DiffDatas.Last().ID1;
            }


            if (LastScanDataId >= 0)
            {
                if (LastScanDataId == mmid.MaxID)
                {
                    //没有数据更新
                    IsError = false;
                    Msg = $"获取{Gage1Title} 数据成功, 但没有数据更新";
                    return false;
                }
                else if (LastScanDataId > mmid.MaxID)
                {
                    //之前的数据库被删除了
                    LastScanDataId = fromId - 1;
                    fromId -= (Mix - 1);
                }
                else
                {
                    //混合的数据也在这里获取！！！
                    //从这个Id开始获取
                    fromId = Math.Min(fromId, LastScanDataId + 1 - (Mix - 1));
                }
            }
            else {
                LastScanDataId = fromId - 1;
                fromId -= (Mix - 1);
            }

            //---------------------------------------------------------------------------------------------
            commandText = $"SELECT * FROM ScanData WHERE [ID] >= { fromId } ORDER BY [ID]";

            dataTable = await ExecuteReader2(sqliteRemoteService1, commandText, $"获取{Gage1Title}");
            if (dataTable == null)
                return false;


            List<Db_ScanData> db_ScanDatas = SQLite.SQLiteHelper.ToObjs<Db_ScanData>(dataTable);
            var lc_ScanDatas1 = ThkLcAutoMapperProfile.Mapper.Map<List<Lc_ScanData>>(db_ScanDatas);
            if (lc_ScanDatas1.Count == 0)
            {
                IsError = true;
                ErrMsg = $"获取{Gage1Title} 数据异常:不能翻译出任何数据";
                logger.Error(ErrMsg);
                return false;
            }

            //计算全部混合后的数据
            scanDataDiffPacks = new List<ScanDataDiffPack>();

            for (int i = 0; i < lc_ScanDatas1.Count(); i++)
            {
                int index = lc_ScanDatas1.Count() - i - 1;
                var lc_ScanData = lc_ScanDatas1[index];
                if (lc_ScanData.ID <= LastScanDataId)
                {
                    //这个已经添加到数据库了，不需要再操作, 后面的也删除
                    break;
                }
                double[] datamix1 = CalMix(lc_ScanDatas1, index);
                ScanDataDiffPack scanDataDiffPack = new ScanDataDiffPack()
                {
                    scanData1 = lc_ScanData,
                    datamix1 = datamix1
                };
                scanDataDiffPacks.Add(scanDataDiffPack);
            }
            //反转
            scanDataDiffPacks.Reverse();
            return true;
        }
        async Task<bool> OnPoll_getScanData0()
        {
            DataTable dataTable;
            //---------------------------------------------------------------------------------------------
            //获取前机数据
            DateTime beginTime1 = scanDataDiffPacks.First().scanData1.Time;
            DateTime endTime1 = scanDataDiffPacks.Last().scanData1.Time;
            //使用当前的速度就算了

            double velocity = scanDataDiffPacks.Max(sddp => sddp.scanData1.Velocity);
            if (velocity < MinVelocity)
            {
                //没有动起来！！！！
                //TODO
                return false;
            }
            DistanceTs = TimeSpan.FromMinutes(Distance / velocity);
            var beginTime0 = beginTime1 - DistanceTs - TimeSpan.FromSeconds(DistanceTs.TotalSeconds * 0.1);
            var endTime0 = endTime1 - DistanceTs + TimeSpan.FromSeconds(DistanceTs.TotalSeconds * 0.1);


            commandText = $"SELECT * FROM ScanData WHERE [Time] >= {beginTime0.ToStringOfSQLiteFieldType()} AND [Time] <= {endTime0.ToStringOfSQLiteFieldType()} ORDER BY [ID]";
            dataTable = await ExecuteReader2(sqliteRemoteService0, commandText, $"获取{Gage0Title}");
            if (dataTable == null)
                return false;


            var db_ScanDatas = SQLite.SQLiteHelper.ToObjs<Db_ScanData>(dataTable);
            var lc_ScanDatas0 = ThkLcAutoMapperProfile.Mapper.Map<List<Lc_ScanData>>(db_ScanDatas);
            if (lc_ScanDatas0.Count == 0)
            {
                IsError = true;
                ErrMsg = $"获取{Gage0Title} 数据异常:不能翻译出任何数据";
                logger.Error(ErrMsg);
                return false;
            }
            bool isFirst = true;
            for (int i = 0; i < scanDataDiffPacks.Count(); i++)
            {
                var lc_ScanData = scanDataDiffPacks[i].scanData1;
                DateTime targetDt = lc_ScanData.Time - DistanceTs;
                int index = FindBestTarget(lc_ScanDatas0, targetDt, SecondTolerance);
                if (index < 0)
                    continue;

                if (isFirst)
                {
                    isFirst = false;
                    //评估从 gage0 获取的数据是否足够计算混合数据
                    int fromIndex = index - Mix + 1;
                    if (fromIndex < 0)
                    {
                        //获取更多数据
                        var fromId = lc_ScanDatas0.First().ID + fromIndex;
                        var toId = lc_ScanDatas0.First().ID - 1;
                        var lc_ScanDatas = await GetScanData0(fromId, toId);
                        if (lc_ScanDatas != null)
                        {
                            lc_ScanDatas0.InsertRange(0, lc_ScanDatas);
                            index += lc_ScanDatas.Count();
                        }
                    }
                }

                scanDataDiffPacks[i].scanData0 = lc_ScanDatas0[index];

                //求出混合数据
                var datamix0 = CalMix(lc_ScanDatas0, index);
                scanDataDiffPacks[i].datamix0 = datamix0;
            }
            //把找不到 gage0 数据的，删除
            scanDataDiffPacks.RemoveAll(sddp => sddp.scanData0 == null);

            return true;
        }
        async Task<List<Lc_ScanData>> GetScanData0(Int64 fromId, Int64 toId) 
        {
            //获取更多数据
            commandText = $"SELECT * FROM ScanData WHERE [ID] >= {fromId} AND [ID] <= {toId} ORDER BY [ID]";
            DataTable dataTable;
            try
            {
                dataTable = await ExecuteReader(sqliteRemoteService0, commandText);
            }
            catch (Exception e)
            {
                //IsError = true;
                //ErrMsg = $"获取{Gage0Title} 数据异常: {commandText} {e.Message}";
                //logger.Error(ErrMsg);
                return null;
            }

            if (dataTable.Rows.Count > 0)
            {
                var db_ScanDatas = SQLite.SQLiteHelper.ToObjs<Db_ScanData>(dataTable);
                var lc_ScanDatas = ThkLcAutoMapperProfile.Mapper.Map<List<Lc_ScanData>>(db_ScanDatas);
                if (lc_ScanDatas.Count == 0)
                {
                    //IsError = true;
                    //ErrMsg = $"获取{Gage0Title} 数据异常:不能翻译出任何数据";
                    //logger.Error(ErrMsg);
                    return null;
                }
                return lc_ScanDatas;
            }
            return null;
        }
        void OnPoll_calDiff() 
        {
            for (int i = 0; i < scanDataDiffPacks.Count(); i++)
            {

                var lc_ScanData0 = scanDataDiffPacks[i].scanData0;
                var lc_ScanData1 = scanDataDiffPacks[i].scanData1;


                //把 lc_ScanData0 与 lc_ScanData1 中间对齐，以 lc_ScanData1为准

                //宽度小于10mm 肯定没有数据
                if (lc_ScanData1.FilmEnd - lc_ScanData1.FilmBegin < 10)
                    continue;

                int filmMid1 = (lc_ScanData1.FilmEnd + lc_ScanData1.FilmBegin) / 2;

                //宽度小于10mm 肯定没有数据
                if (lc_ScanData0.FilmEnd - lc_ScanData0.FilmBegin < 10)
                    continue;

                int filmMid0 = (lc_ScanData0.FilmEnd + lc_ScanData0.FilmBegin) / 2;

                //gage0 极片偏移
                int filmOffset0 = filmMid1 - filmMid0;
                scanDataDiffPacks[i].scanData0.FilmBegin += filmOffset0;
                scanDataDiffPacks[i].scanData0.FilmEnd += filmOffset0;


                //gage0 数据偏移
                int dataOffset0 = (int)Math.Round(filmOffset0 / 10.0);

                //求出混合数据
                var datamix0 = scanDataDiffPacks[i].datamix0;

                //把gage0 也偏移了!!!!!!
                scanDataDiffPacks[i].datamix0 = Move(datamix0, dataOffset0, scanDataDiffPacks[i].datamix1.Count());


                //求出偏差 lc_ScanData1 - lc_ScanData0 
                scanDataDiffPacks[i].datadiff = CalDiff(
                    scanDataDiffPacks[i].datamix1,
                    scanDataDiffPacks[i].datamix0);
            }
        }
        async Task<bool> OnPoll_getProfile() 
        {
            DataTable dataTable;
            //获取Profile
            commandText = $"SELECT * FROM Profile" +
                $" WHERE [ID] >= {scanDataDiffPacks.First().scanData1.ProfileID}" +
                $" AND [ID] <= {scanDataDiffPacks.Last().scanData1.ProfileID}";
            //获取数据对应的 Db_Profile
            dataTable = await ExecuteReader2(sqliteRemoteService1, commandText, $"获取{Gage1Title}");
            if (dataTable == null)
                return false;

            db_Profiles1 = SQLite.SQLiteHelper.ToObjs<Db_Profile>(dataTable);

            commandText = $"SELECT * FROM BoltmapData" +
                $" WHERE [ProfileID] >= {db_Profiles1.First().ID}" +
                $" AND [ProfileID] <= {db_Profiles1.Last().ID}";
            //获取数据对应的 Db_Profile
            dataTable = await ExecuteReader2(sqliteRemoteService1, commandText, $"获取{Gage1Title}");
            if (dataTable == null)
                return false;

            var db_BoltmapDatas1 = SQLite.SQLiteHelper.ToObjs<Db_BoltmapData>(dataTable);
            lc_BoltmapDatas1 = ThkLcAutoMapperProfile.Mapper.Map< List < Db_BoltmapData > ,List <Lc_BoltmapData>>(db_BoltmapDatas1);

            //获取Profile
            commandText = $"SELECT * FROM Profile" +
                $" WHERE [ID] >= {scanDataDiffPacks.First().scanData0.ProfileID}" +
                $" AND [ID] <= {scanDataDiffPacks.Last().scanData0.ProfileID}";
            //获取数据对应的 Db_Profile
            dataTable = await ExecuteReader2(sqliteRemoteService0, commandText, $"获取{Gage0Title}");
            if (dataTable == null)
                return false;

            db_Profiles0 = SQLite.SQLiteHelper.ToObjs<Db_Profile>(dataTable);

            return true;
        }
        void OnPoll_save() 
        {
            Lc_DiffProfile currProfile = null;
            //从本地数据库，获取最后一条 profile
            var db_DiffProfiles = dbModel.TbProfile.Last(1);
            if (db_DiffProfiles.Count() > 0) {
                currProfile = DiffLcAutoMapperProfile.Mapper.Map<Lc_DiffProfile>(db_DiffProfiles.Last());
            }


            List<string> sqls = new List<string>();

            for (int i = 0; i < scanDataDiffPacks.Count(); i++)
            {
                var scanDataDiffPack = scanDataDiffPacks[i];

                Lc_DiffData lc_DiffData = new Lc_DiffData()
                {
                    Time = scanDataDiffPack.scanData1.Time,
                    K = scanDataDiffPack.scanData1.K,
                    B = scanDataDiffPack.scanData1.B,
                    IsND = scanDataDiffPack.scanData1.IsND,
                    Position = scanDataDiffPack.scanData1.Position,
                    Velocity = scanDataDiffPack.scanData1.Velocity,
                    FilmBegin = scanDataDiffPack.scanData1.FilmBegin,
                    FilmEnd = scanDataDiffPack.scanData1.FilmEnd,
                    Datas = Round(scanDataDiffPack.datadiff,2),

                    ID0 = scanDataDiffPack.scanData0.ID,
                    ID1 = scanDataDiffPack.scanData1.ID,
                    Time0 = scanDataDiffPack.scanData0.Time,
                    K0 = scanDataDiffPack.scanData0.K,
                    B0 = scanDataDiffPack.scanData0.B,
                    Datas0 = Round(scanDataDiffPack.datamix0,2),
                    Datas1 = Round(scanDataDiffPack.datamix1,2),

                    Avg = Round(scanDataDiffPack.datadiff.AverageNoNull(),2),
                    Avg0 = Round(scanDataDiffPack.datamix0.AverageNoNull(), 2),
                    Avg1 = Round(scanDataDiffPack.datamix1.AverageNoNull(), 2)
                };



                var profile0 = db_Profiles0.Find(p => p.ID == scanDataDiffPack.scanData0.ProfileID);
                if (profile0 == null)
                {
                    //异常
                    continue;
                }

                var profile1 = db_Profiles1.Find(p => p.ID == scanDataDiffPack.scanData1.ProfileID);
                if (profile1 == null)
                {
                    //异常
                    continue;
                }
                var boltmapData1 = lc_BoltmapDatas1.Find(b => b.ProfileID == profile1.ID);
                if (boltmapData1 == null)
                {
                    //异常
                    continue;
                }
                Lc_DiffProfile lc_DiffProfile = new Lc_DiffProfile()
                {
                    PName = profile1.PName,
                    Batch = profile1.Batch,
                    Number = profile1.Number,
                    BSD = profile1.BSD,
                    StartTime = profile1.StartTime,
                    EndTime = profile1.EndTime,
                    FilmLength = profile1.FilmLength,
                    IsFinished = profile1.IsFinished,
                    Target = profile1.Target - profile0.Target,
                    Tolerance = profile1.Tolerance,
                    CtrlLine = profile1.CtrlLine,
                    SRBegin = profile1.SRBegin,
                    SREnd = profile1.SREnd,
                    FilmWidth = boltmapData1.FilmWidth,
                    Map = boltmapData1.Map,

                    Target0 = profile0.Target,
                    Tolerance0 = profile0.Tolerance,
                    CtrlLine0 = profile0.CtrlLine,

                    Target1 = profile1.Target,
                    Tolerance1 = profile1.Tolerance,
                    CtrlLine1 = profile1.CtrlLine
                };

                bool isProfileNew = false;
                if (currProfile == null)
                {
                    isProfileNew = true;
                }
                else {
                    if (currProfile.IsFinished)
                    {
                        //之前的结束了需要新建
                        isProfileNew = true;
                    }
                    else if (currProfile.StartTime != profile1.StartTime)
                    {
                        //之前的开始时间与现在的不一样，需要新建
                        isProfileNew = true;
                    }
                    else { 
                        //参数不一样，需要更新
                    }
                }
                if (isProfileNew)
                {
                    currProfile = lc_DiffProfile;
                    currProfile.ID = dbModel.TbProfile.FreeID;
                    var db_DiffProfile = DiffLcAutoMapperProfile.Mapper.Map<Db_DiffProfile>(currProfile);
                    sqls.Add(SQLiteHelper.GetInsertCommandText(db_DiffProfile));
                }
                else {
                    lc_DiffProfile.ID = currProfile.ID;
                    currProfile = lc_DiffProfile;
                    var db_DiffProfile = DiffLcAutoMapperProfile.Mapper.Map<Db_DiffProfile>(currProfile);
                    sqls.Add(SQLiteHelper.GetUpdateCommandText(db_DiffProfile, $"WHERE ID={db_DiffProfile.ID}"));
                }

                lc_DiffData.ID = dbModel.TbDiffData.FreeID;
                lc_DiffData.ProfileID = currProfile.ID;
                var db_DiffData = DiffLcAutoMapperProfile.Mapper.Map<Db_DiffData>(lc_DiffData);
                sqls.Add(SQLiteHelper.GetInsertCommandText(db_DiffData));

                LastScanDataId = lc_DiffData.ID1;
            }

            dbModel.sqliteHelper.QueryTranAsync(sqls);

        }
        double[] Round(double[] datas, int decimals) {
            double[] datas2 = new double[datas.Count()];
            for (int i = 0; i < datas.Count(); i++) {
                if (!double.IsNaN(datas[i]))
                {
                    datas2[i] = Math.Round(datas[i], decimals);
                }
                else {
                    datas2[i] = double.NaN;
                }
            }
            return datas2;
        }
        double Round(double data, int decimals) {
            if (!double.IsNaN(data))
            {
                return Math.Round(data, decimals);
            }
            else
            {
                return double.NaN;
            }
        }
        class ScanDataDiffPack
        {
            public Lc_ScanData scanData1=null;
            public Lc_ScanData scanData0=null;
            public double[] datadiff=null;
            public double[] datamix0=null;
            public double[] datamix1=null;
        }
        
        double[] CalDiff(double[] datas1, double[] datas0, int datas0_offset) 
        {
            double[] datadiff = new double[datas1.Count()];
            for (int i = 0; i < datadiff.Count(); i++)
            {
                int index0 = i + datas0_offset;
                if (index0 < 0)
                {
                    datadiff[i] = double.NaN;
                }
                else if (index0 >= datas0.Count())
                {
                    datadiff[i] = double.NaN;
                }
                else if (double.IsNaN(datas1[i]) || double.IsNaN(datas0[index0]))
                {
                    datadiff[i] = double.NaN;
                }
                else {
                    datadiff[i] = datas1[i] - datas0[index0];
                }
            }
            return datadiff;
        }

        double[] CalDiff(double[] datas1, double[] datas0)
        {
            double[] datadiff = new double[datas1.Count()];
            for (int i = 0; i < datadiff.Count(); i++)
            {
                if (double.IsNaN(datas1[i]) || double.IsNaN(datas0[i]))
                {
                    datadiff[i] = double.NaN;
                }
                else
                {
                    datadiff[i] = datas1[i] - datas0[i];
                }
            }
            return datadiff;
        }

        /// <summary>
        /// 移动datas_org 数组,到新的尺寸。  当0位置 需要移动到 10位置， offset=10
        /// </summary>
        /// <param name="datas_org"></param>
        /// <param name="offset"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        double[] Move(double[] datas_org, int offset, int size) 
        {
            double[] datas_new = new double[size];
            for (int i = 0; i < datas_new.Count(); i++) {
                int index = i - offset;
                if (index < 0)
                    datas_new[i] = double.NaN;
                else if(index>= datas_org.Count())
                    datas_new[i] = double.NaN;
                else
                    datas_new[i] = datas_org[index];
            }
            return datas_new;
        }



        /// <summary>
        /// 找出在扫描队列中，时间最接近的数据，偏差不能大于 secTol
        /// </summary>
        /// <param name="lc_ScanDatas"></param>
        /// <param name="targetDt"></param>
        /// <returns></returns>
        int FindBestTarget(List<Lc_ScanData> lc_ScanDatas, DateTime targetDt, double secTol) 
        {
            double min = double.MaxValue;
            int index = -1;
            for (int i = 0; i < lc_ScanDatas.Count(); i++) {
                var lc_ScanData = lc_ScanDatas[i];
                double s = Math.Abs((lc_ScanData.Time - targetDt).TotalSeconds);
                if (s < min)
                {
                    min = s;
                    index = i;
                }
                else if (s > min) 
                {
                    break;
                }
            }
            if (min > secTol)
                return -1;
            return index;
        }
        double[] CalMix(List<Lc_ScanData> lc_ScanDatas, int index) 
        {
            double[] sum = new double[lc_ScanDatas[index].Datas.Count()];
            int[] cnt = new int[sum.Count()];

            for (int i = 0; i < Mix; i++)
            {
                int index2 = index - i;
                if (index2 < 0)//已经没数据了
                    break;

                var lc_ScanData = lc_ScanDatas[index2];
                if (lc_ScanData.Datas.Count() != sum.Count())
                    break;//异常

                for (int k = 0; k < sum.Count(); k++)
                {
                    double d = lc_ScanData.Datas[k];
                    if (!double.IsNaN(d))
                    {
                        sum[k] += d;
                        cnt[k]++;
                    }
                }
            }
            for (int i = 0; i < sum.Count(); i++) {
                if (cnt[i] > 0)
                    sum[i] = sum[i] / cnt[i];
                else
                    sum[i] = double.NaN;
            }
            return sum;
        }



        public void Stop()
        {
            if (!IsRunning)
            {
                return;
            }
            cancellationTokenSource.Cancel();
            IsRunning = false;
            
        }

        double Average(IEnumerable<double> array)
        {
            var list2 = array.Where((d) =>
            {
                if (double.IsNaN(d))
                    return false;
                else
                    return true;
            });
            if (list2.Count() == 0)
                return double.NaN;

            return list2.Average();
        }
        public bool Load()
        {
            try
            {

                string json = File.ReadAllText("ddgen.json");
                Newtonsoft.Json.JsonConvert.PopulateObject(json, this);
                return true;
            }
            catch
            {
                return false;
            }
        }
        public bool Save()
        {
            try
            {
                var jsondb = DDGenJsonDb.Mapper.Map<DDGenJsonDb>(this);

                File.WriteAllText("ddgen.json", Newtonsoft.Json.JsonConvert.SerializeObject(jsondb, Newtonsoft.Json.Formatting.Indented));
                return true;
            }
            catch {
                return false;
            }
        }
    }

    
    public class DDGenJsonDb 
    {
        public static Mapper Mapper { get; } = new AutoMapper.Mapper(new MapperConfiguration(c =>
        {
            c.CreateMap<DDGen, DDGenJsonDb>();
        }));

        /// <summary>
        /// 前机设备
        /// </summary>
        public string Addr0 = "127.0.0.1:20000";
        /// <summary>
        /// 后机设备
        /// </summary>
        public string Addr1  = "127.0.0.1:20000";

        /// <summary>
        /// 前后机距离，单位m
        /// </summary>
        public double Distance = 40;

        /// <summary>
        /// 混合数
        /// </summary>
        public int Mix = 6;


    }
}
