﻿using Misc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DoubleDataGenerate.Core.Model
{
    /// <summary>
    /// 数据库 扫描产品表
    /// </summary>
    public class Lc_DiffProfile
    {
        #region 正常profile
        public Int64 ID { get; set; }

        /// <summary>
        /// 产品名称
        /// </summary>
        public string PName { get; set; }

        /// <summary>
        /// 批号
        /// </summary>
        public string Batch { get; set; }

        /// <summary>
        /// 卷号
        /// </summary>
        public string Number { get; set; }

        /// <summary>
        /// 0极片，1单面，2双面
        /// </summary>
        public int BSD { get; set; }

        /// <summary>
        /// 开始生产时间
        /// </summary>
        public DateTime StartTime { get; set; }

        /// <summary>
        /// 结束生产时间
        /// </summary>
        public DateTime EndTime { get; set; }

        /// <summary>
        /// 生产长度
        /// </summary>
        public double FilmLength { get; set; }

        /// <summary>
        /// 生产完成
        /// </summary>
        public bool IsFinished { get; set; }

        /// <summary>
        /// 目标值
        /// </summary>
        public double Target { get; set; }

        /// <summary>
        /// 规格线
        /// </summary>
        public double Tolerance { get; set; }

        /// <summary>
        /// 控制线
        /// </summary>
        public double CtrlLine { get; set; }

        /// <summary>
        /// 扫描开始位置 cm
        /// </summary>
        public int SRBegin { get; set; }


        /// <summary>
        /// 扫描结束位置 cm
        /// </summary>
        public int SREnd { get; set; }
        #endregion
        #region 分条
        /// <summary>
        /// 膜宽，mm， profile设定的膜宽
        /// </summary>
        public int FilmWidth { get; set; }

        /// <summary>
        /// Range[] 通过 json 转为的string
        /// </summary>
        public Range[] Map { get; set; }
        #endregion
        #region 机头


        /// <summary>
        /// 机头目标值
        /// </summary>
        public double Target0 { get; set; }

        /// <summary>
        /// 机头规格线
        /// </summary>
        public double Tolerance0 { get; set; }

        /// <summary>
        /// 机头控制线
        /// </summary>
        public double CtrlLine0 { get; set; }
        #endregion
        #region 机尾


        /// <summary>
        /// 机尾目标值
        /// </summary>
        public double Target1 { get; set; }

        /// <summary>
        /// 机尾规格线
        /// </summary>
        public double Tolerance1 { get; set; }

        /// <summary>
        /// 机尾控制线
        /// </summary>
        public double CtrlLine1 { get; set; }
        #endregion
    }

    
    public class Lc_DiffData
    {
        #region 正常ScanData

        public Int64 ID { get; set; }

        public Int64 ProfileID { get; set; }

        /// <summary>
        /// 扫描时间
        /// </summary>
        public DateTime Time { get; set; }

        public double K { get; set; }

        public double B { get; set; }

        /// <summary>
        /// 探头行进的方向 为反方向
        /// </summary>
        public bool IsND { get; set; }


        /// <summary>
        /// 纵向位置 m, 小数点后2位
        /// </summary>
        public double Position { get; set; }

        /// <summary>
        /// 速度 m/min
        /// </summary>
        public double Velocity { get; set; }

        /// <summary>
        /// 膜开始位置 mm
        /// </summary>
        public int FilmBegin { get; set; }

        /// <summary>
        /// 膜结束位置 mm
        /// </summary>
        public int FilmEnd { get; set; }

        /// <summary>
        /// 数据
        /// </summary>
        public double[] Datas { get; set; }
        #endregion


        #region 机头机尾数据

        /// <summary>
        /// 机头扫描ScanDataID
        /// </summary>
        public Int64 ID0 { get; set; }

        /// <summary>
        /// 机尾扫描ScanDataID
        /// </summary>
        public Int64 ID1 { get; set; }

        /// <summary>
        /// 机头扫描时间
        /// </summary>
        public DateTime Time0 { get; set; }

        /// <summary>
        /// 机头K
        /// </summary>
        public double K0 { get; set; }

        /// <summary>
        /// 机头B
        /// </summary>
        public double B0 { get; set; }

        /// <summary>
        /// 机头原始数据
        /// </summary>
        public double[] Datas0 { get; set; }


        /// <summary>
        /// 机尾原始数据
        /// </summary>
        public double[] Datas1 { get; set; }

        /// <summary>
        /// 差分的均值
        /// </summary>
        public double Avg { get; set; }
        /// <summary>
        /// 机头的均值
        /// </summary>
        public double Avg0 { get; set; }
        /// <summary>
        /// 机尾的均值
        /// </summary>
        public double Avg1 { get; set; }
        #endregion
    }
}
