﻿using SQLite;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DoubleDataGenerate.Core.Model
{
    #region 表

    /// <summary>
    /// 数据库 扫描产品表
    /// </summary>
    [Table("Profile")]
    public class Db_Profile : INotifyPropertyChanged
    {
        [Key]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }
        [PropertyIndex(1)]
        public string PName { get; set; }
        [PropertyIndex(2)]
        public string Batch { get; set; }
        [PropertyIndex(3)]
        public string Number { get; set; }
        [PropertyIndex(4)]
        public int BSD { get; set; }
        [PropertyIndex(5)]
        public DateTime StartTime { get; set; }
        [PropertyIndex(6)]
        public DateTime EndTime { get; set; }
        [PropertyIndex(7)]
        public double FilmLength { get; set; }
        [PropertyIndex(8)]
        public bool IsFinished { get; set; }
        [PropertyIndex(9)]
        public double Target { get; set; }
        [PropertyIndex(10)]
        public double Tolerance { get; set; }
        [PropertyIndex(11)]
        public double CtrlLine { get; set; }
        [PropertyIndex(12)]
        public int SRBegin { get; set; }
        [PropertyIndex(13)]
        public int SREnd { get; set; }


        public event PropertyChangedEventHandler PropertyChanged;
    }


    [Table("ScanData")]
    public class Db_ScanData
    {
        [Key]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }

        [PropertyIndex(1)]
        public Int64 ProfileID { get; set; }

        /// <summary>
        /// 扫描时间
        /// </summary>
        [PropertyIndex(2)]
        public DateTime Time { get; set; }

        [PropertyIndex(3)]
        public double K { get; set; }

        [PropertyIndex(4)]
        public double B { get; set; }

        /// <summary>
        /// 探头行进的方向 为反方向
        /// </summary>
        [PropertyIndex(5)]
        public bool IsND { get; set; }


        /// <summary>
        /// 纵向位置 m, 小数点后2位
        /// </summary>
        [PropertyIndex(6)]
        public double Position { get; set; }

        /// <summary>
        /// 速度 m/min
        /// </summary>
        [PropertyIndex(7)]
        public double Velocity { get; set; }

        /// <summary>
        /// 膜开始位置 mm
        /// </summary>
        [PropertyIndex(8)]
        public int FilmBegin { get; set; }

        /// <summary>
        /// 膜结束位置 mm
        /// </summary>
        [PropertyIndex(9)]
        public int FilmEnd { get; set; }

        /// <summary>
        /// 数据
        /// </summary>
        [PropertyIndex(10)]
        public string Datas { get; set; }
    }

    /// <summary>
    /// 数据库 扫描中的温修AD，样品AD
    /// </summary>
    [Table("SampleData")]
    public class Db_SampleData
    {
        [Key]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }
        [PropertyIndex(1)]
        public DateTime Time { get; set; }
        [PropertyIndex(2)]
        public string SampleADs { get; set; }
    }

    [Table("BoltmapData")]
    public class Db_BoltmapData
    {
        /// <summary>
        /// ID
        /// </summary>
        [Key]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }

        /// <summary>
        /// DB_Profile 的 ID
        /// </summary>
        [PropertyIndex(1)]
        public Int64 ProfileID { get; set; }

        /// <summary>
        /// 膜宽，mm， profile设定的膜宽
        /// </summary>
        [PropertyIndex(2)]
        public int FilmWidth { get; set; }

        /// <summary>
        /// Range[] 通过 json 转为的string
        /// </summary>
        [PropertyIndex(3)]
        public string Map { get; set; }
    }
    #endregion
}
