﻿using AutoMapper;
using Misc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DoubleDataGenerate.Core.Model
{
    /// <summary>
    /// LC 与 DB 类的映射关系， 会在程序入口处， 手动使用 
    /// var assemblies = System.AppDomain.CurrentDomain.GetAssemblies();
    /// var cfg = new MapperConfigurationExpression();
    /// cfg.AddMaps(assemblies);
    /// Mapper.Initialize(cfg);
    /// 
    /// 枚举全部程序集 中的AutoMapper.Profile 全部加载！！！
    /// </summary>
    public class ThkLcAutoMapperProfile
    {
        public static Mapper Mapper { get; } = new AutoMapper.Mapper(new MapperConfiguration(c =>
        {
            #region LC_SampleData

            c.CreateMap<Lc_SampleData, Db_SampleData>().ForMember(s => s.SampleADs, opt =>
            {
                opt.MapFrom(s => Newtonsoft.Json.JsonConvert.SerializeObject(s.SampleADs));
            }).ReverseMap().ForMember(s => s.SampleADs, opt =>
            {
                opt.MapFrom(s => Newtonsoft.Json.JsonConvert.DeserializeObject<int[]>(s.SampleADs));
            });

            #endregion

            #region LC_ScanData
            c.CreateMap<Lc_ScanData, Db_ScanData>().ForMember(s => s.Datas, opt =>
            {
                opt.MapFrom(s => Newtonsoft.Json.JsonConvert.SerializeObject(s.Datas));
            }).ReverseMap().ForMember(s => s.Datas, opt =>
            {
                opt.MapFrom(s => Newtonsoft.Json.JsonConvert.DeserializeObject<double[]>(s.Datas));
            });
            c.CreateMap<Lc_ScanData, Lc_ScanData>().ForMember(s => s.Datas, opt =>
            {
                opt.MapFrom(s => s.Datas.Clone());
            });
            #endregion

            #region LC_BoltmapData
            c.CreateMap<Lc_BoltmapData, Db_BoltmapData>().ForMember(s => s.Map, opt =>
            {
                opt.MapFrom(s => Newtonsoft.Json.JsonConvert.SerializeObject(s.Map));
            }).ReverseMap().ForMember(s => s.Map, opt =>
            {
                opt.MapFrom(s => Newtonsoft.Json.JsonConvert.DeserializeObject<Range[]>(s.Map));
            });
            #endregion
        }));


    }
}
