﻿using DoubleDataGenerate.Core;
using GalaSoft.MvvmLight.Command;
using MahApps.Metro.Controls;
using MahApps.Metro.Controls.Dialogs;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;

namespace DoubleDataGenerate
{
    public class WdSettingVm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        #region 参数
        /// <summary>
        /// 前机设备
        /// </summary>
        public string Addr0 { get; set; } = "127.0.0.1:20000";
        /// <summary>
        /// 后机设备
        /// </summary>
        public string Addr1 { get; set; } = "127.0.0.1:20000";

        /// <summary>
        /// 前后机距离，单位m
        /// </summary>
        public double Distance { get; set; } = 40;

        /// <summary>
        /// 最小线速度 2m/min
        /// </summary>
        public double MinVelocity { get; set; } = 3;

        /// <summary>
        /// 找位置的时间差，不能大于60s
        /// </summary>
        public int SecondTolerance { get; set; } = 60;
        private int mix = 6;
        /// <summary>
        /// 混合数
        /// </summary>
        public int Mix
        {
            get { return mix; }
            set
            {
                if (value < 1)
                    value = 1;

                if (mix != value)
                    mix = value;
            }

        }

        /// <summary>
        /// 轮询后机数据库周期！！！ 单位min 默认1，最小值0.5
        /// </summary>
        public int IntervalMin { get; set; } = 1;

        #endregion

        public RelayCommand OKCmd { get; private set; }

        Core.DDGen dDGen;
        MetroWindow metroWindow;
        public void Init(MetroWindow metroWindow)
        {
            InitCmd();
            this.metroWindow = metroWindow;
            dDGen = Core.DDGen.Instance;
            Addr0 = dDGen.Addr0;
            Addr1 = dDGen.Addr1;
            Distance = dDGen.Distance;
            MinVelocity = dDGen.MinVelocity;
            SecondTolerance = dDGen.SecondTolerance;
            Mix = dDGen.Mix;
            IntervalMin = dDGen.IntervalMin;
        }
        void InitCmd()
        {
            OKCmd = new RelayCommand(OK);
        }
        async void OK()
        {
            //检查参数是否正确
            Regex regex = new Regex(@"\A\d+.\d+.\d+.\d+:\d+\z");
            if (!regex.IsMatch(Addr0))
            {
                //异常
                await metroWindow.ShowMessageAsync(
                    "参数异常",
                    $"{DDGen.Gage0Title} 地址异常,格式为xxx.xxx.xxx.xxx:xxxx,\n" +
                    $"例如: 192.168.1.20:20000");
            }
            else if (!regex.IsMatch(Addr1))
            {
                //异常
                await metroWindow.ShowMessageAsync(
                    "参数异常",
                    $"{DDGen.Gage1Title} 地址异常,格式为xxx.xxx.xxx.xxx:xxxx,\n" +
                    $"例如: 127.0.0.1:20000");
            }
            else if (Distance < 10)
            {
                //异常
                await metroWindow.ShowMessageAsync(
                    "参数异常",
                    $"{DDGen.Gage1Title} 与 {DDGen.Gage0Title} 的距离应该大于 10m");
            }
            else if (MinVelocity < 1)
            {
                //异常
                await metroWindow.ShowMessageAsync(
                    "参数异常",
                    $"最小线速度应该大于 1m/min");
            }
            else if (SecondTolerance < 10)
            {
                //异常
                await metroWindow.ShowMessageAsync(
                    "参数异常",
                    $"找位置的时间差应该大于10s");
            }
            else if (Mix < 3)
            {
                //异常
                await metroWindow.ShowMessageAsync(
                    "参数异常",
                    $"混合数 应该大于3");
            }
            else if (IntervalMin < 0) {
                //异常
                await metroWindow.ShowMessageAsync(
                    "参数异常",
                    $"轮询时间间隔应该大于0min");
            }

            bool isAddrChanged = (dDGen.Addr0 != Addr0) || (dDGen.Addr1 != Addr1);

            dDGen.Addr0 = Addr0;
            dDGen.Addr1 = Addr1;
            dDGen.Distance = Distance;
            dDGen.MinVelocity = MinVelocity;
            dDGen.SecondTolerance = SecondTolerance;
            dDGen.Mix = Mix;
            dDGen.IntervalMin = IntervalMin;

            if (dDGen.Save())
            {
                string msg = "";
                if (isAddrChanged) {
                    msg = "地址被修改，需要重启软件才能生效";
                }
                
                await metroWindow.ShowMessageAsync("",$"设置成功.{msg}");
                return;
            }
            else
            {
                await metroWindow.ShowMessageAsync("", $"设置失败.");
                return;
            }
        }
    }
}
